// test if logic rules
const {negationIntroduction, 
    negationElimination,
    andIntroduction, 
    andRightElimination,
    andLeftElimination,
    orRightIntroduction, 
    orLeftIntroduction, 
    orElimination,
    implyIntroduction,
    implyElimination,
    contradiction } = require('../../src/utils/ndRules');
const { parse } = require("../../src/utils/parser");


// negationIntroduction Rule Test
test('negationIntroduction should return the correct premises for valid "not" conclusion', () => {

    const input = parse("|- !A");

    const output = negationIntroduction(input);

    expect(output).toEqual(parse("A |- false"));
});


test('negationIntroduction should throw an error for invalid conclusion type', () => {

    const input = parse("|- A");

    expect(() => negationIntroduction(input)).toThrow("The conclusion must be of type 'not'");
});


//negationElimination
test('negationElimination should return the correct premises for valid "false" conclusion', () => {

    const input = parse("|- false");
    const userInput = parse("A");

    const output = negationElimination(input, userInput);

    expect(output).toEqual([parse("|- A"), parse("|- !A")]);
});

test('negationElimination should throw an error for invalid conclusion type', () => {
    const input = parse("|- !A & B");

    expect(() => negationElimination(input)).toThrow("The conclusion must be of type 'false'");
});

// andIntroduction Rule Test
test('andIntroduction should return the correct premises for valid "and" conclusion', () => {

    const input = parse("|- A & B");

    const output = andIntroduction(input);
    const premiseL = output[0];
    const premiseR = output[1];

    expect(premiseL).toEqual(parse("|- A"));
    expect(premiseR).toEqual(parse("|- B"));
});

test('andIntroduction should return the correct premises for valid "and" conclusion', () => {

    const input = parse("|- !A & B");

    const output = andIntroduction(input);
    const premiseL = output[0];
    const premiseR = output[1];
    
    expect(premiseL).toEqual(parse("|- !A"));
    expect(premiseR).toEqual(parse("|- B"));
});

test('andIntroduction should throw an error for invalid conclusion type', () => {
    const input = parse("|- !A || B");

    expect(() => andIntroduction(input)).toThrow("The conclusion must be of type 'and'");
});

// andRightElimination test
test('andRightElimination should return the correct premises', () => {

    const input = parse("|- A");
    const userInput = parse("B");

    const output = andRightElimination(input, userInput);
    
    expect(output).toEqual(parse("|- A & B"));
});

// andLeftElimination test
test('andLeftElimination should return the correct premises', () => {

    const input = parse("|- B");
    const userInput = parse("A");

    const output = andLeftElimination(input, userInput);
    
    expect(output).toEqual(parse("|- A & B"));
});

// orRightIntroduction Rule Test
test('OrRightIntroduction should return the correct premises for valid "or" conclusion', () => {

    const input = parse("|- A || B");

    const output = orRightIntroduction(input);

    expect(output).toEqual(parse("|- A"));
});

test('OrRightIntroduction should return the correct premises for valid "or" conclusion', () => {

    const input = parse("|- !A || B");

    const output = orRightIntroduction(input);

    expect(output).toEqual(parse("|- !A"));
});

test('OrRightIntroduction should return the correct premises for valid "or" conclusion', () => {

    const input = parse("|- (!A-> C) || B & D");

    const output = orRightIntroduction(input);

    expect(output).toEqual(parse("|- !A-> C"));
});

test('OrRightIntroduction should throw an error for invalid conclusion type', () => {
    const input = parse("|- !A & B");

    expect(() => orRightIntroduction(input)).toThrow("The conclusion must be of type 'or'");
});

// OrLeftIntroduction Rule Test
test('OrLeftIntroduction should return the correct premises for valid "or" conclusion', () => {

    const input = parse("|- A || B");

    const output = orLeftIntroduction(input);

    expect(output).toEqual(parse("|- B"));
});

test('OrLeftIntroduction should return the correct premises for valid "or" conclusion', () => {

    const input = parse("|- A || B & D");

    const output = orLeftIntroduction(input);

    expect(output).toEqual(parse("|- B & D"));
});

test('OrLeftIntroduction should throw an error for invalid conclusion type', () => {
    const input = parse("|- !A & B");

    expect(() => orLeftIntroduction(input)).toThrow("The conclusion must be of type 'or'");
});

// orElimination tests
test('orElimination should return the correct premises', () => {

    const input = parse("|- C");
    const userInput1 = parse("A");
    const userInput2 = parse("B");

    const output = orElimination(input, userInput1, userInput2);

    expect(output).toEqual([parse("|- A || B"), parse("A |- C"), parse("B |- C")]);
});


// ImplyIntroduction Rule Test
test('ImplyIntroduction should return the correct premises for valid "imply" conclusion', () => {

    const input = parse("|- A -> B");

    const output = implyIntroduction(input);

    expect(output).toEqual(parse("A |- B"));
});

test('ImplyIntroduction should return the correct premises for valid "imply" conclusion', () => {

    const input = parse("|- !A & C -> B || D");

    const output = implyIntroduction(input);

    expect(output).toEqual(parse("!A & C |-  B || D"));
});

test('ImplyIntroduction should throw an error for invalid conclusion type', () => {
    const input = parse("|- !A & B");

    expect(() => implyIntroduction(input)).toThrow("The conclusion must be of type 'imply'");
});

// implyElimination Rule Test
test('implyElimination should return the correct premises', () => {

    const input = parse("|- B");
    const userInput = parse("A");

    const output = implyElimination(input, userInput);

    expect([output[0], output[1]]).toEqual([parse("|- A -> B"), parse("|- A")]);
});

//contradiction
test('Contradiction rule should return the correct premises', () => {

    const input = parse("|- A");

    const output = contradiction(input);

    expect(output).toEqual(parse("!A |- false"));
});